/*
 * PostfixEval.java
 
 */

package datastructures;

import java.io.*;
import java.util.StringTokenizer;
/**
 *
 * @author  hs, partially based on code by M.A. Weiss
 */
public class PostfixEval {
    
    static final int EOL    = 0;  
    static final int WHITESP= 1;
    static final int VALUE  = 2; 
    static final int MULT   = 3; 
    static final int DIV    = 4; 
    static final int PLUS   = 5; 
    static final int MINUS  = 6; 
    
    
    /** Creates a new instance of PostfixEval */
    public PostfixEval() {
        opST = new DoubleStack();
    }
    
    private DoubleStackIF opST;
    private StringTokenizer str;
    private double val;         // Operand values from the Input string
    
    public double eval(String s) {
        str = new StringTokenizer(s, "+-*/ ",true);
        double result = 0;
        
        while (str.hasMoreTokens()){
            int tokenT  = tokenTyp (str.nextToken());
            if (tokenT != WHITESP & tokenT != EOL) processToken(tokenT );
        }
        
        try {
            result = opST.topAndPop();
        } catch (Underflow e) {
            System.out.println("Syntax error - missing Operand");
            return 0;
        }
        if (!opST.isEmpty()) throw new Underflow("Syntax error - too many operands"); 
        return result; 
    }       
    
    private int tokenTyp(String t) {
        if (t.equals ( " ") )   return WHITESP;
        if (t.equals ( "*") )   return MULT;
        if (t.equals ( "/") )   return DIV;
        if (t.equals ( "+") )   return PLUS;
        if (t.equals ( "-") )   return MINUS;
        try { 
            val = Double.parseDouble(t);
        } catch (NumberFormatException e) { 
            System.err.println("not a number"); 
            return EOL;
        }       
        return VALUE;
    }
    
    private void processToken(int tt) {
        if (tt==WHITESP) ;
        else { 
            if (tt==VALUE) opST.push (val);
            else {
                double right= opST.topAndPop();       
                double left = opST.topAndPop();
                switch (tt) {
                    case MULT  :  opST.push (left * right); 
                                  break;
                    case PLUS  :  opST.push (left+ right); 
                                  break;
                    case DIV   :  if (right == 0) System.err.println ("Division by zero");
                                  else opST.push (left / right); 
                                  break;
                    case MINUS : opST.push (left - right); 
                }
            } 
        }
    }
    
}